@@ -1,16 +1,19 @@
-from typing import Dict, Sequence, Optional
+from typing import Dict, Sequence, Optional, Callable
 
 import mypy.subtypes
 import mypy.sametypes
 from mypy.expandtype import expand_type
-from mypy.types import Type, TypeVarId, TypeVarType, CallableType, AnyType, PartialType
-from mypy.messages import MessageBuilder
+from mypy.types import (
+    Type, TypeVarId, TypeVarType, CallableType, AnyType, PartialType, get_proper_types
+)
 from mypy.nodes import Context
 
 
-def apply_generic_arguments(callable: CallableType, orig_types: Sequence[Optional[Type]],
-                            msg: MessageBuilder, context: Context,
-                            skip_unsatisfied: bool = False) -> CallableType:
+def apply_generic_arguments(
+        callable: CallableType, orig_types: Sequence[Optional[Type]],
+        report_incompatible_typevar_value: Callable[[CallableType, Type, str, Context], None],
+        context: Context,
+        skip_unsatisfied: bool = False) -> CallableType:
     """Apply generic type arguments to a callable type.
 
     For example, applying [int] to 'def [T] (T) -> T' results in
@@ -25,10 +28,10 @@ def apply_generic_arguments(callable: CallableType, orig_types: Sequence[Optiona
     assert len(tvars) == len(orig_types)
     # Check that inferred type variable values are compatible with allowed
     # values and bounds.  Also, promote subtype values to allowed values.
-    types = list(orig_types)
+    types = get_proper_types(orig_types)
     for i, type in enumerate(types):
         assert not isinstance(type, PartialType), "Internal error: must never apply partial type"
-        values = callable.variables[i].values
+        values = get_proper_types(callable.variables[i].values)
         if type is None:
             continue
         if values:
@@ -55,16 +58,16 @@ def apply_generic_arguments(callable: CallableType, orig_types: Sequence[Optiona
                 if skip_unsatisfied:
                     types[i] = None
                 else:
-                    msg.incompatible_typevar_value(callable, type, callable.variables[i].name,
-                                                   context)
+                    report_incompatible_typevar_value(callable, type, callable.variables[i].name,
+                                                      context)
         else:
             upper_bound = callable.variables[i].upper_bound
             if not mypy.subtypes.is_subtype(type, upper_bound):
                 if skip_unsatisfied:
                     types[i] = None
                 else:
-                    msg.incompatible_typevar_value(callable, type, callable.variables[i].name,
-                                                   context)
+                    report_incompatible_typevar_value(callable, type, callable.variables[i].name,
+                                                      context)
 
     # Create a map from type variable id to target type.
     id_to_type = {}  # type: Dict[TypeVarId, Type]
