@@ -106,7 +106,6 @@ def main(script_path: Optional[str],
     if MEM_PROFILE:
         from mypy.memprofile import print_memory_profile
         print_memory_profile()
-    del res  # Now it's safe to delete
 
     code = 0
     if messages:
@@ -129,6 +128,9 @@ def main(script_path: Optional[str],
     elif code:
         sys.exit(code)
 
+    # HACK: keep res alive so that mypyc won't free it before the hard_exit
+    list([res])
+
 
 # Make the help output a little less jarring.
 class AugmentedHelpFormatter(argparse.RawDescriptionHelpFormatter):
@@ -216,7 +218,7 @@ def infer_python_executable(options: Options,
     python_executable = special_opts.python_executable or options.python_executable
 
     if python_executable is None:
-        if not special_opts.no_executable:
+        if not special_opts.no_executable and not options.no_site_packages:
             python_executable = _python_executable_from_version(options.python_version)
     options.python_executable = python_executable
 
@@ -837,7 +839,7 @@ def process_options(args: List[str],
     except PythonExecutableInferenceError as e:
         parser.error(str(e))
 
-    if special_opts.no_executable:
+    if special_opts.no_executable or options.no_site_packages:
         options.python_executable = None
 
     # Paths listed in the config file will be ignored if any paths are passed on
