@@ -265,7 +265,8 @@ class MessageBuilder:
                     format_type(original_type)), context, code=codes.INDEX)
         elif member == '__setitem__':
             # Indexed set.
-            self.fail('Unsupported target for indexed assignment', context, code=codes.INDEX)
+            self.fail('Unsupported target for indexed assignment ({})'.format(
+                format_type(original_type)), context, code=codes.INDEX)
         elif member == '__call__':
             if isinstance(original_type, Instance) and \
                     (original_type.type.fullname == 'builtins.function'):
@@ -732,6 +733,9 @@ class MessageBuilder:
             self.fail('Too many values to unpack ({} expected, {} provided)'.format(
                 expected, provided), context)
 
+    def unpacking_strings_disallowed(self, context: Context) -> None:
+        self.fail("Unpacking a string is disallowed", context)
+
     def type_not_iterable(self, type: Type, context: Context) -> None:
         self.fail('\'{}\' object is not iterable'.format(type), context)
 
@@ -768,6 +772,14 @@ class MessageBuilder:
                   .format(arg_num, name, target, arg_type_in_supertype_f),
                   context,
                   code=codes.OVERRIDE)
+        self.note(
+            'This violates the Liskov substitution principle',
+            context,
+            code=codes.OVERRIDE)
+        self.note(
+            'See https://mypy.readthedocs.io/en/stable/common_issues.html#incompatible-overrides',
+            context,
+            code=codes.OVERRIDE)
 
         if name == "__eq__" and type_name:
             multiline_msg = self.comparison_method_example_msg(class_name=type_name)
@@ -813,14 +825,6 @@ class MessageBuilder:
             self.fail('Type application has too few types ({} expected)'
                       .format(expected_arg_count), context)
 
-    def alias_invalid_in_runtime_context(self, item: ProperType, ctx: Context) -> None:
-        kind = (' to Callable' if isinstance(item, CallableType) else
-                ' to Tuple' if isinstance(item, TupleType) else
-                ' to Union' if isinstance(item, UnionType) else
-                ' to Literal' if isinstance(item, LiteralType) else
-                '')
-        self.fail('The type alias{} is invalid in runtime context'.format(kind), ctx)
-
     def could_not_infer_type_arguments(self, callee_type: CallableType, n: int,
                                        context: Context) -> None:
         callee_name = callable_name(callee_type)
@@ -1054,7 +1058,7 @@ class MessageBuilder:
             self.note(line, context)
 
     def unsupported_type_type(self, item: Type, context: Context) -> None:
-        self.fail('Unsupported type Type[{}]'.format(format_type(item)), context)
+        self.fail('Cannot instantiate type "Type[{}]"'.format(format_type_bare(item)), context)
 
     def redundant_cast(self, typ: Type, context: Context) -> None:
         self.fail('Redundant cast to {}'.format(format_type(typ)), context,
@@ -1146,6 +1150,9 @@ class MessageBuilder:
                 item_name, format_item_name_list(typ.items.keys())), context)
         else:
             self.fail("TypedDict {} has no key '{}'".format(format_type(typ), item_name), context)
+            matches = best_matches(item_name, typ.items.keys())
+            if matches:
+                self.note("Did you mean {}?".format(pretty_seq(matches[:3], "or")), context)
 
     def typeddict_context_ambiguous(
             self,
