@@ -119,11 +119,9 @@ You can use `api.options.new_semantic_analyzer` to check whether the new
 semantic analyzer is enabled (it's always true in mypy 0.730 and later).
 """
 
-import types
-
-from abc import abstractmethod, abstractproperty
+from abc import abstractmethod
 from typing import Any, Callable, List, Tuple, Optional, NamedTuple, TypeVar, Dict
-from mypy_extensions import trait
+from mypy_extensions import trait, mypyc_attr
 
 from mypy.nodes import (
     Expression, Context, ClassDef, SymbolTableNode, MypyFile, CallExpr
@@ -134,7 +132,6 @@ from mypy.messages import MessageBuilder
 from mypy.options import Options
 from mypy.lookup import lookup_fully_qualified
 from mypy.errorcodes import ErrorCode
-import mypy.interpreted_plugin
 
 
 @trait
@@ -182,7 +179,7 @@ AnalyzeTypeContext = NamedTuple(
         ('api', TypeAnalyzerPluginInterface)])
 
 
-@trait
+@mypyc_attr(allow_interpreted_subclasses=True)
 class CommonPluginApi:
     """
     A common plugin API (shared between semantic analysis and type checking phases)
@@ -217,7 +214,8 @@ class CheckerPluginInterface:
     path = None  # type: str
 
     # Type context for type inference
-    @abstractproperty
+    @property
+    @abstractmethod
     def type_context(self) -> List[Optional[Type]]:
         """Return the type context of the plugin"""
         raise NotImplementedError
@@ -351,7 +349,8 @@ class SemanticAnalyzerPluginInterface:
         """
         raise NotImplementedError
 
-    @abstractproperty
+    @property
+    @abstractmethod
     def final_iteration(self) -> bool:
         """Is this the final iteration of semantic analysis?"""
         raise NotImplementedError
@@ -445,6 +444,7 @@ DynamicClassDefContext = NamedTuple(
     ])
 
 
+@mypyc_attr(allow_interpreted_subclasses=True)
 class Plugin(CommonPluginApi):
     """Base class of all type checker plugins.
 
@@ -683,74 +683,6 @@ class Plugin(CommonPluginApi):
 T = TypeVar('T')
 
 
-class WrapperPlugin(Plugin):
-    """A plugin that wraps an interpreted plugin.
-
-    This is a ugly workaround the limitation that mypyc-compiled
-    classes can't be subclassed by interpreted ones, so instead we
-    create a new class for interpreted clients to inherit from and
-    dispatch to it from here.
-
-    Eventually mypyc ought to do something like this automatically.
-    """
-
-    def __init__(self, plugin: mypy.interpreted_plugin.InterpretedPlugin) -> None:
-        super().__init__(plugin.options)
-        self.plugin = plugin
-
-    def set_modules(self, modules: Dict[str, MypyFile]) -> None:
-        self.plugin.set_modules(modules)
-
-    def lookup_fully_qualified(self, fullname: str) -> Optional[SymbolTableNode]:
-        return self.plugin.lookup_fully_qualified(fullname)
-
-    def report_config_data(self, ctx: ReportConfigContext) -> Any:
-        return self.plugin.report_config_data(ctx)
-
-    def get_additional_deps(self, file: MypyFile) -> List[Tuple[int, str, int]]:
-        return self.plugin.get_additional_deps(file)
-
-    def get_type_analyze_hook(self, fullname: str
-                              ) -> Optional[Callable[[AnalyzeTypeContext], Type]]:
-        return self.plugin.get_type_analyze_hook(fullname)
-
-    def get_function_hook(self, fullname: str
-                          ) -> Optional[Callable[[FunctionContext], Type]]:
-        return self.plugin.get_function_hook(fullname)
-
-    def get_method_signature_hook(self, fullname: str
-                                  ) -> Optional[Callable[[MethodSigContext], CallableType]]:
-        return self.plugin.get_method_signature_hook(fullname)
-
-    def get_method_hook(self, fullname: str
-                        ) -> Optional[Callable[[MethodContext], Type]]:
-        return self.plugin.get_method_hook(fullname)
-
-    def get_attribute_hook(self, fullname: str
-                           ) -> Optional[Callable[[AttributeContext], Type]]:
-        return self.plugin.get_attribute_hook(fullname)
-
-    def get_class_decorator_hook(self, fullname: str
-                                 ) -> Optional[Callable[[ClassDefContext], None]]:
-        return self.plugin.get_class_decorator_hook(fullname)
-
-    def get_metaclass_hook(self, fullname: str
-                           ) -> Optional[Callable[[ClassDefContext], None]]:
-        return self.plugin.get_metaclass_hook(fullname)
-
-    def get_base_class_hook(self, fullname: str
-                            ) -> Optional[Callable[[ClassDefContext], None]]:
-        return self.plugin.get_base_class_hook(fullname)
-
-    def get_customize_class_mro_hook(self, fullname: str
-                                     ) -> Optional[Callable[[ClassDefContext], None]]:
-        return self.plugin.get_customize_class_mro_hook(fullname)
-
-    def get_dynamic_class_hook(self, fullname: str
-                               ) -> Optional[Callable[[DynamicClassDefContext], None]]:
-        return self.plugin.get_dynamic_class_hook(fullname)
-
-
 class ChainedPlugin(Plugin):
     """A plugin that represents a sequence of chained plugins.
 
@@ -831,18 +763,3 @@ class ChainedPlugin(Plugin):
             if hook:
                 return hook
         return None
-
-
-def _dummy() -> None:
-    """Only used to test whether we are running in compiled mode."""
-
-
-# This is an incredibly frumious hack. If this module is compiled by mypyc,
-# set the module 'Plugin' attribute to point to InterpretedPlugin. This means
-# that anything interpreted that imports Plugin will get InterpretedPlugin
-# while anything compiled alongside this module will get the real Plugin.
-if isinstance(_dummy, types.BuiltinFunctionType):
-    plugin_types = (Plugin, mypy.interpreted_plugin.InterpretedPlugin)  # type: Tuple[type, ...]
-    globals()['Plugin'] = mypy.interpreted_plugin.InterpretedPlugin
-else:
-    plugin_types = (Plugin,)
