@@ -1,10 +1,12 @@
 """Utility functions with no non-trivial dependencies."""
+
 import os
 import pathlib
 import re
 import subprocess
 import sys
-import os
+import hashlib
+import io
 
 from typing import (
     TypeVar, List, Tuple, Optional, Dict, Sequence, Iterable, Container, IO, Callable
@@ -142,7 +144,7 @@ def read_py_file(path: str, read: Callable[[str], bytes],
     """
     try:
         source = read(path)
-    except (IOError, OSError):
+    except OSError:
         return None
     else:
         try:
@@ -424,9 +426,12 @@ def get_terminal_width() -> int:
     """Get current terminal width if possible, otherwise return the default one."""
     try:
         cols, _ = os.get_terminal_size()
-        return cols
     except OSError:
         return DEFAULT_COLUMNS
+    else:
+        if cols == 0:
+            return DEFAULT_COLUMNS
+        return cols
 
 
 def soft_wrap(msg: str, max_len: int, first_offset: int,
@@ -466,6 +471,18 @@ def soft_wrap(msg: str, max_len: int, first_offset: int,
     return padding.join(lines)
 
 
+def hash_digest(data: bytes) -> str:
+    """Compute a hash digest of some data.
+
+    We use a cryptographic hash because we want a low probability of
+    accidental collision, but we don't really care about any of the
+    cryptographic properties.
+    """
+    # Once we drop Python 3.5 support, we should consider using
+    # blake2b, which is faster.
+    return hashlib.sha256(data).hexdigest()
+
+
 class FancyFormatter:
     """Apply color and bold font to terminal output.
 
@@ -527,7 +544,16 @@ class FancyFormatter:
         if not CURSES_ENABLED:
             return False
         try:
-            curses.setupterm()
+            # setupterm wants a fd to potentially write an "initialization sequence".
+            # We override sys.stdout for the daemon API so if stdout doesn't have an fd,
+            # just give it /dev/null.
+            try:
+                fd = sys.stdout.fileno()
+            except io.UnsupportedOperation:
+                with open("/dev/null", "rb") as f:
+                    curses.setupterm(fd=f.fileno())
+            else:
+                curses.setupterm(fd=fd)
         except curses.error:
             # Most likely terminfo not found.
             return False
@@ -604,8 +630,11 @@ class FancyFormatter:
                 return (loc + self.style('error:', 'red', bold=True) +
                         self.highlight_quote_groups(msg))
             codepos = msg.rfind('[')
-            code = msg[codepos:]
-            msg = msg[:codepos]
+            if codepos != -1:
+                code = msg[codepos:]
+                msg = msg[:codepos]
+            else:
+                code = ""  # no error code specified
             return (loc + self.style('error:', 'red', bold=True) +
                     self.highlight_quote_groups(msg) + self.style(code, 'yellow'))
         elif ': note:' in error:
@@ -672,3 +701,8 @@ class FancyFormatter:
         if not use_color:
             return msg
         return self.style(msg, 'red', bold=True)
+
+
+def is_typeshed_file(file: str) -> bool:
+    # gross, but no other clear way to tell
+    return 'typeshed' in os.path.abspath(file).split(os.sep)
